@@ -99,13 +99,24 @@ correlated_RemoveToLimit <- function(data,unitPvalues,limit=0,thr=0.975,maxLoops
 univariate_Logit <- function(data=NULL, Outcome=NULL, pvalue=0.2, adjustMethod="BH", uniTest=c("zIDI","zNRI"),limit=0,...,n = 0)
 {
 	varlist <-colnames(data);
-    if (inherits(data[,Outcome],"factor")) data[,Outcome] <- as.numeric(as.character(data[,Outcome]));
+#    if (inherits(data[,Outcome],"factor")) data[,Outcome] <- as.numeric(as.character(data[,Outcome]));
 	varlist <- varlist[Outcome != varlist];
+	nvar <- length(varlist);
 	varlist <- cbind(varlist,varlist);
 	uniTest <- match.arg(uniTest);
-	univ <- ForwardSelection.Model.Bin(nrow(varlist),1.0,0.0,1,"1",Outcome,varlist,data,1,type="LOGIT",selectionType=uniTest);
-	unitPvalues <- 2.0*(1.0 - pnorm(univ$base.Zvalues));
-	unitPvalues[unitPvalues > 1.0] <- 1.0;
+	outcomes <- data[,Outcome];
+	outlist <- unique(outcomes)
+	pvalue <- pvalue/sqrt(length(outlist) - 1.0)
+	unitPvalues <- rep(1.0,nvar);
+	for (oft in outlist[-1])
+	{
+		datat <- rbind(subset(data,outcomes == oft),subset(data,outcomes != oft))
+		datat[,Outcome] <- 1*(datat[,Outcome] == oft);
+		univ <- ForwardSelection.Model.Bin(nrow(varlist),1.0,0.0,1,"1",Outcome,varlist,datat,1,type="LOGIT",selectionType=uniTest);
+		pval <- 2.0*(1.0 - pnorm(univ$base.Zvalues));
+		pval[pval > 1.0] <- 1.0;
+		unitPvalues <- pmin(unitPvalues,pval);
+	}
 
 	names(unitPvalues) <-  varlist[,1];
 	unitPvalues <- unitPvalues[order(unitPvalues)];
@@ -133,15 +144,29 @@ univariate_Logit <- function(data=NULL, Outcome=NULL, pvalue=0.2, adjustMethod="
 univariate_residual <- function(data=NULL, Outcome=NULL, pvalue=0.2, adjustMethod="BH",uniTest=c("Ftest","Binomial","Wilcox","tStudent"),type=c("LM","LOGIT"),limit=0,...,n = 0)
 {
 	varlist <- colnames(data);
-    if (inherits(data[,Outcome],"factor")) data[,Outcome] <- as.numeric(as.character(data[,Outcome]));
+#    if (inherits(data[,Outcome],"factor")) data[,Outcome] <- as.numeric(as.character(data[,Outcome]));
 	varlist <- varlist[Outcome != varlist];
+	nvar <- length(varlist);
 	varlist <- cbind(varlist,varlist)
 	uniTest <- match.arg(uniTest);
 	type <- match.arg(type);
 
-	univ <- ForwardSelection.Model.Res(nrow(varlist),1.0,0.0,1,"1",Outcome,varlist,data,1,type=type,testType=uniTest);
-	unitPvalues <- 2.0*(1.0 - pnorm(univ$base.Zvalues));
-	unitPvalues[unitPvalues > 1.0] <- 1.0;
+	outcomes <- data[,Outcome];
+	outlist <- unique(outcomes)
+	pvalue <- pvalue/sqrt(length(outlist) - 1.0)
+	unitPvalues <- rep(1.0,nvar);
+	for (oft in outlist[-1])
+	{
+		datat <- rbind(subset(data,outcomes == oft),subset(data,outcomes != oft))
+		datat[,Outcome] <- 1*(datat[,Outcome] == oft);
+		univ <- ForwardSelection.Model.Res(nrow(varlist),1.0,0.0,1,"1",Outcome,varlist,datat,1,type=type,testType=uniTest);
+		pval <- 2.0*(1.0 - pnorm(univ$base.Zvalues));
+		pval[pval > 1.0] <- 1.0;
+		unitPvalues <- pmin(unitPvalues,pval);
+	}
+
+
+
 #	print(unitPvalues);
 	names(unitPvalues) <- varlist[,1];
 	unitPvalues <- unitPvalues[order(unitPvalues)];
@@ -228,7 +253,7 @@ univariate_filter <- function(data=NULL, Outcome=NULL, pvalue=0.2,pvalueMethod=w
 
 	outcomes <- as.character(data[,Outcome]);
 	outlist <- unique(outcomes)
-	pvalue <- pvalue/(length(outlist) - 1.0)
+	pvalue <- pvalue/sqrt(length(outlist) - 1.0)
 	for (oft in outlist[-1])
 	{
 		case <- subset(data,outcomes == oft);
@@ -283,48 +308,7 @@ univariate_KS <- function(data=NULL, Outcome=NULL, pvalue=0.2, adjustMethod="BH"
 										limit=limit,
 										...,
 										n = n)
-	# varlist <-colnames(data);
-    # if (inherits(data[,Outcome], "factor")) data[,Outcome] <- as.numeric(as.character(data[,Outcome]));
-	# case <- subset(data,get(Outcome) == 1);
-	# control <- subset(data,get(Outcome) == 0);
-	# varlist <- varlist[Outcome != varlist];
-	# unitPvalues <- numeric(length(varlist));
-	# names(unitPvalues) <- varlist;
-	
-	# for (j in varlist) 
-	# {
-		# tb <- table(data[,j],data[,Outcome])
-		# if (nrow(tb) > 5)
-		# {
-			 # pval <- ks.test(control[,j],case[,j],na.action = na.exclude)$p.value; 
-		# }
-		# else
-		# {
-			# pval <- prop.test(tb)$p.value
-		# }
-		# if (inherits(pval, "try-error")) {pval <- 1.0;}
-		# if (is.null(pval)) { pval <- 1.0; }
-		# if (is.na(pval)) {pval <- 1.0;}
-		# unitPvalues[j] <- pval;
-	# }
-  	# unitPvalues <- unitPvalues[order(unitPvalues)];
-  	# unadjusted <- unitPvalues;
-	# unitPvalues <- p.adjust(unitPvalues,adjustMethod,n=max(n,length(unitPvalues)));
-	# top <- unitPvalues[1];
-	# if ((top < 0.45) && (unadjusted[1] < 0.05))
-	# {
-		# top <- unitPvalues[unitPvalues <= 1.01*top];
-	# }
-	# unitPvalues <- unitPvalues[unitPvalues <= pvalue];
-	# if (length(unitPvalues) > 1) 
-	# {
-		# unitPvalues <- correlated_RemoveToLimit(data,unitPvalues,limit,...);
-	# }
-	# else 
-	# {
-		# unitPvalues <- top;
-	# }
-	# attr(unitPvalues,"Unadjusted") <- unadjusted;
+
 
    return(unitPvalues);
 }
@@ -342,41 +326,6 @@ univariate_Wilcoxon <- function(data=NULL, Outcome=NULL, pvalue=0.2, adjustMetho
 										...,
 										n = n)
 
-	# varlist <-colnames(data);
-    # if (inherits(data[,Outcome], "factor")) data[,Outcome] <- as.numeric(as.character(data[,Outcome]));
-	# case <- subset(data,get(Outcome) == 1);
-	# control <- subset(data,get(Outcome) == 0);
-	# varlist <- varlist[Outcome != varlist];
-	# unitPvalues <- numeric(length(varlist));
-	# names(unitPvalues) <- varlist;
-
-	# for (j in varlist) 
-	# {
-		 # pval <- wilcox.test(control[,j],case[,j],na.action = na.exclude)$p.value; 
-		 # if (inherits(pval, "try-error")) {pval <- 1.0;}
-		 # if (is.null(pval)) { pval <- 1.0; }
-		 # if (is.na(pval)) {pval <- 1.0;}
-		 # unitPvalues[j] <- pval;
-	# }
-  
-	# unitPvalues <- unitPvalues[order(unitPvalues)];
-  	# unadjusted <- unitPvalues;
-	# unitPvalues <- p.adjust(unitPvalues,adjustMethod,n=max(n,length(unitPvalues)));
-	# top <- unitPvalues[1];
-	# if ((top < 0.45) && (unadjusted[1] < 0.05))
-	# {
-		# top <- unitPvalues[unitPvalues <= 1.01*top];
-	# }
-	# unitPvalues <- unitPvalues[unitPvalues <= pvalue];
-	# if (length(unitPvalues) > 1) 
-	# {
-		# unitPvalues <- correlated_RemoveToLimit(data,unitPvalues,limit,...);
-	# }
-	# else 
-	# {
-		# unitPvalues <- top;
-	# }
-   # attr(unitPvalues,"Unadjusted") <- unadjusted;
    return(unitPvalues);
 }
 
@@ -392,39 +341,6 @@ univariate_tstudent <- function(data=NULL, Outcome=NULL, pvalue=0.2, adjustMetho
 										...,
 										n = n)
 
-	# varlist <-colnames(data);
-    # if (inherits(data[,Outcome], "factor")) data[,Outcome] <- as.numeric(as.character(data[,Outcome]));
-	# case <- subset(data,get(Outcome) == 1);
-	# control <- subset(data,get(Outcome) == 0);
-	# varlist <- varlist[Outcome != varlist];
-	# unitPvalues <- numeric(length(varlist));
-	# names(unitPvalues) <- varlist;
-	# for (j in varlist) 
-	# {
-		# pval <- try(t.test(control[,j],case[,j],na.action = na.exclude)$p.value);
-		# if (inherits(pval, "try-error")) {pval <- 1.0;}
-		# if (is.null(pval)) { pval <- 1.0; }
-		# if (is.na(pval)) {pval <- 1.0;}
-		# unitPvalues[j] <-  pval;	
-	# }
-	# unitPvalues <- unitPvalues[order(unitPvalues)];
-  	# unadjusted <- unitPvalues;
-	# unitPvalues <- p.adjust(unitPvalues,adjustMethod,n=max(n,length(unitPvalues)));
-	# top <- unitPvalues[1];
-	# if ((top < 0.45) && (unadjusted[1] < 0.05))
-	# {
-		# top <- unitPvalues[unitPvalues <= 1.01*top];
-	# }
-	# unitPvalues <- unitPvalues[unitPvalues <= pvalue];
-	# if (length(unitPvalues) > 1) 
-	# {
-		# unitPvalues <- correlated_RemoveToLimit(data,unitPvalues,limit,...);
-	# }
-	# else 
-	# {
-		# unitPvalues <- top;
-	# }
-   # attr(unitPvalues,"Unadjusted") <- unadjusted;
     return(unitPvalues);
 }
 
@@ -440,7 +356,7 @@ univariate_correlation <- function(data=NULL, Outcome=NULL, pvalue=0.2, adjustMe
 	{
 		outcomes <- as.character(data[,Outcome]);
 		outlist <- unique(outcomes);
-		pvalue <- pvalue/(length(outlist) - 1.0)
+		pvalue <- pvalue/sqrt(length(outlist) - 1.0);
 		for (ots in outlist[-1])
 		{
 			for (j in varlist) 
@@ -539,7 +455,7 @@ univariate_BinEnsemble <- function(data,Outcome,pvalue=0.2,limit=0,adjustMethod=
   names(varcount) <- colnames(data);
   names(rankVar) <- colnames(data);
 
-  if (inherits(data[,Outcome], "factor")) data[,Outcome] <- as.numeric(as.character(data[,Outcome]));
+#  if (inherits(data[,Outcome], "factor")) data[,Outcome] <- as.numeric(as.character(data[,Outcome]));
   
   data <- data[,c(Outcome,correlated_Remove(data,colnames(data)[!(colnames(data) %in% Outcome)]))]
 
@@ -569,7 +485,7 @@ univariate_BinEnsemble <- function(data,Outcome,pvalue=0.2,limit=0,adjustMethod=
 
 
 	pvaltest <- univariate_residual(data,Outcome,pvalue=pvalue,limit=-1,uniTest="tStudent",type="LOGIT",adjustMethod=adjustMethod);
-#cat("->tstudent")
+	#cat("->tstudent")
 	 geomMeanpVal <- geomMeanpVal*(attr(pvaltest,"Unadjusted")[names(geomMeanpVal)]);
 	 maxPval <- pmax(maxPval,attr(pvaltest,"Unadjusted")[names(geomMeanpVal)]);
 	 minPval <- pmin(minPval,attr(pvaltest,"Unadjusted")[names(geomMeanpVal)]);
@@ -592,45 +508,12 @@ univariate_BinEnsemble <- function(data,Outcome,pvalue=0.2,limit=0,adjustMethod=
   both <- pmin(pvaltest[features],allf[features]);
   allf <- c(pvaltest[!(names(pvaltest) %in% features)],allf[!(names(allf) %in% features)],both);
 
+# Removing the largest p-value for the estimation of the geometric mean p-value
   expgeom <- 1.0/(length(pvallist)-1);
-  maxPval[maxPval < 1.0e-12] <- 1.0e-12;
+  maxPval[maxPval < 1.0e-16] <- 1.0e-16;
   geomMeanpVal <- (geomMeanpVal/maxPval)^expgeom;
 
-#  print(names(allf))
-  
-  limitmrmr = length(allf) + 1;
-  if ((limit > 0) && (limit < 1.0))
-  {
-	limitmrmr = min(length(allf),limit*(ncol(data)-1));
-  } 
-  else if (limit > 2)
-  {
-	limitmrmr = min((ncol(data)-1),limit);
-  }
-
-  if (limitmrmr < 2 ) limitmrmr = 2;
-  
-  selfeat <- names(geomMeanpVal[geomMeanpVal<0.1]);
-  if (length(selfeat) <= limitmrmr)
-  {
-  	selfeat <- names(geomMeanpVal[order(geomMeanpVal)])[1:min(limitmrmr+1,(ncol(data)-1))];
-  }
-  mRMRf <- mRMR.classic_FRESA(data[,c(Outcome,selfeat)],Outcome,feature_count = limitmrmr)
-#  cat("->mRMRf")
-   
-  mRMRf <- mRMRf[mRMRf > 0];
-  varcount[names(mRMRf)] <- varcount[names(mRMRf)] + 1;
-  rankVar[names(mRMRf)] <- rankVar[names(mRMRf)] + c(1:length(mRMRf));
-  pvallist$mRMR <- mRMRf;
-  if (length(mRMRf) > 0)
-  {
-    missing <- !(names(mRMRf) %in% names(allf))
-    allf <- c(allf,mRMRf[missing])
-	powgeom <- 1.0/expgeom;
-	expgeom <- 1.0/(powgeom + 1.0);
-	geomMeanpVal[names(mRMRf)] <- ((geomMeanpVal[names(mRMRf)]^powgeom)*minPval[names(mRMRf)])^expgeom; # Adjusting mRMR selected features
-  }
-
+# Sorting and top selection
   geomMeanpVal <- geomMeanpVal[order(geomMeanpVal-varcount[names(geomMeanpVal)])];
   allf <- p.adjust(geomMeanpVal,adjustMethod);
   adjusptedp <- allf;
@@ -785,7 +668,7 @@ ClassMetric95ci <- function(datatest,nss=4000)
 			{
 				cpre[n] <- NA;
 			}
-			if (is.na(csen) || is.na(cpre))
+			if (is.na(csen[n]) || is.na(cpre[n]))
 			{
 				cF1[n] <- NA;
 			}
