@@ -84,6 +84,8 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
     baseFeatures <- character()
   }
   AbaseFeatures <- character()
+  bfeat <- unique(c(baseFeatures,AbaseFeatures))
+  
   addedlist <- 1;
   lp = 0;
   uncorrelatedFetures <- character();
@@ -139,27 +141,29 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
   {
       if (!is.null(Outcome))
       {
-          outcomep <- univariate_correlation(data[,c(Outcome,varincluded)],Outcome,method=method,limit=0,pvalue=2*unipvalue,thr = 0.999*thr) # the top associated features to the outcome
-          if (length(baseFeatures)==0) baseFeatures <- names(outcomep);
+          outcomep <- univariate_correlation(data[,c(Outcome,varincluded)],Outcome,method=method,limit=0,pvalue=0.5*unipvalue,thr = 0.75*thr) # the top associated features to the outcome
           asociatedtoOutcome <- attr(outcomep,"Unadjusted")
           asociatedtoOutcome <- p.adjust(asociatedtoOutcome,"BH"); # adjusting for false discovery the unadjusted pvalues
-          asociatedtoOutcome <- names(asociatedtoOutcome[asociatedtoOutcome < unipvalue])
+          asociatedtoOutcome <- names(asociatedtoOutcome[asociatedtoOutcome < 0.5*unipvalue])
+          if (length(baseFeatures)==0) 
+          {
+            baseFeatures <- names(outcomep);
+            baseFeatures <- baseFeatures[baseFeatures %in% asociatedtoOutcome];
+            baseFeatures <- as.character(correlated_Remove(cormat,baseFeatures,thr = 0.75*thr,isDataCorMatrix=TRUE))
+          }
           outcomep <- NULL;
     #      print(asociatedtoOutcome)
       }
-      else
-      {
-         maxcor <- apply(cormat,2,max)
-         AbaseFeatures <- varincluded;
-         names(AbaseFeatures) <- AbaseFeatures;
-         ordcor <- maxcor
-         cormat[cormat < thr] <- 0;
-         ordcor <- 0.95*maxcor + 
-                      0.02*(apply(cormat,2,sum)/(0.001 + apply(1*(cormat >= thr),2,sum))) +
-                      0.02*apply(cormat,2,mean)
-         AbaseFeatures <- AbaseFeatures[order(-ordcor[AbaseFeatures])];
-         AbaseFeatures <- as.character(correlated_Remove(cormat,AbaseFeatures,thr = thr,isDataCorMatrix=TRUE))
-      }
+       maxcor <- apply(cormat,2,max)
+       AbaseFeatures <- varincluded;
+       names(AbaseFeatures) <- AbaseFeatures;
+       ordcor <- maxcor
+       cormat[cormat < thr] <- 0;
+       ordcor <- 0.95*maxcor + 
+                    0.02*(apply(cormat,2,sum)/(0.001 + apply(1*(cormat >= thr),2,sum))) +
+                    0.02*apply(cormat,2,mean)
+       AbaseFeatures <- AbaseFeatures[order(-ordcor[AbaseFeatures])];
+       AbaseFeatures <- as.character(correlated_Remove(cormat,AbaseFeatures,thr = 0.75*thr,isDataCorMatrix=TRUE))
       unipvalue <- min(unipvalue,2.0*unipvalue*sqrt(totcorr)/totFeatures); ## Adjusting for false association
       bvarincluded <- character();
       baseIncluded <- character();
@@ -175,7 +179,7 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
           {
             bmaxcor <- apply(bcormat,2,max)
           }
-          bvarincluded <- names(bmaxcor)[bmaxcor >= thr];
+          bvarincluded <- names(bmaxcor)[bmaxcor > thr];
           bcormat <- NULL;
         } 
       }
@@ -186,7 +190,10 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
       rownames(DeCorrmatrix) <- varincluded;
 
       cormat <- cormat[varincluded,varincluded];
+      baseFeatures <- baseIncluded;
       bfeat <- unique(c(baseIncluded,AbaseFeatures))
+      bfeat <- as.character(correlated_Remove(cormat,bfeat,thr = 0.75*thr,isDataCorMatrix=TRUE));
+
       thr2 <- thr
       thr <- thr2*1.001;
       wthr <- 0.5 - 0.5*skipRelaxed;
@@ -206,9 +213,10 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
         ordcor <- 0.95*maxcor + 
                   0.02*(apply(cormat,2,sum)/(0.001 + apply(1*(cormat >= thr2),2,sum))) +
                   0.02*apply(cormat,2,mean)
+
         if (length(bfeat) > 0)
         {
-          ordcor[bfeat] <- ordcor[bfeat] + 0.01;
+          ordcor[bfeat] <- ordcor[bfeat] + 10.00;
         }
         
         topfeat <- topfeat[maxcor[topfeat] >= thr];
@@ -220,7 +228,7 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
           rownames(betamatrix) <- varincluded;
           topfeat <- topfeat[order(-ordcor[topfeat])];
           topfeat <- correlated_Remove(cormat,topfeat,thr = thr,isDataCorMatrix=TRUE)
-           intopfeat <- character();
+          intopfeat <- character();
           toBeDecorrelated <- length(topfeat)
           if (verbose)  cat(lp,", Top:",toBeDecorrelated,"<",thr,">");
           maxcortp <- thr;
@@ -238,7 +246,7 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
             for (feat in topfeat)
             {
 #                if (verbose && (feat==topfeat[1]))  cat("<");
-                falive <- varincluded[!(varincluded %in% c(decorrelatedFetureList,bfeat))]
+                falive <- varincluded[!(varincluded %in% decorrelatedFetureList)]
                 tobereviewed <- topfeat[!(topfeat %in% c(featMarked,feat))]
                 if ((length(tobereviewed) > 1) && (length(falive) > 1))
                 {
@@ -250,10 +258,10 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
                   maxcortp <- thr;
                 }
                 corlist <- cormat[,feat];
-                corlist <- corlist[corlist >= maxcortp];
+                corlist <- corlist[corlist > maxcortp];
 
                 varlist <- names(corlist)
-                varlist <- varlist[!(varlist %in% decorrelatedFetureList)]
+                varlist <- varlist[!(varlist %in% unique(c(decorrelatedFetureList,bfeat)))]
                 ovarlist <- varlist;
 
                 if (length(varlist) > 0)
@@ -270,12 +278,11 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
                       }
                       if (length(varlist) == 0)
                       {
-#                        if (verbose && (feat==topfeat[1]))  cat("[",feat %in% asociatedtoOutcome,"]");
-                        varlist <- names(corlist)
-                        varlist <- varlist[!(varlist %in% decorrelatedFetureList)]
+                        if (verbose && (feat==topfeat[1]))  cat("[",feat %in% asociatedtoOutcome,"]");
+                        varlist <- ovarlist;
                       }
                     }
-#                   if (verbose && (feat==topfeat[1]))  cat("(",length(varlist),")");
+                   if (verbose && (feat==topfeat[1]))  cat("(",length(varlist),")");
                    if (useFastCor)
                    {
                       prebetas <- getAllBetaCoefficients(feat,varlist);
@@ -487,7 +494,7 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
             }
           }
 
-          correlatedToBase <- varincluded[varincluded %in% baseFeatures];
+          correlatedToBase <- varincluded[varincluded %in% bfeat];
           if (length(correlatedToBase) > 1)
           {
             correlatedToBase <- apply(DeCorrmatrix[correlatedToBase,] != 0,2,sum)
@@ -500,7 +507,7 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
               correlatedToBase <- varincluded[DeCorrmatrix[correlatedToBase,] != 0]
             }
           }
-          correlatedToBase <- correlatedToBase[!(correlatedToBase %in% baseFeatures)];
+          correlatedToBase <- correlatedToBase[!(correlatedToBase %in% bfeat)];
           if (verbose) 
           {
              if (useFastCor)
@@ -515,15 +522,15 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
               cat ("[",lp,"],",max(cormat),". Cor to Base:",length(correlatedToBase),", ABase:",length(AbaseFeatures),"\n")
           }
           newnames <- colnames(dataAdjusted)
-          newnames[newnames %in% c(AbaseFeatures,baseFeatures)] <- paste("Ba_",newnames[newnames %in% c(AbaseFeatures,baseFeatures)],sep="") 
+          newnames[newnames %in% bfeat] <- paste("Ba_",newnames[newnames %in% bfeat],sep="") 
           newnames[newnames %in% varincluded] <- paste("De_",newnames[newnames %in% varincluded],sep="")
           colnames(dataAdjusted) <- newnames
           newnames <- colnames(DeCorrmatrix)
-          newnames[newnames %in% c(AbaseFeatures,baseFeatures)] <- paste("Ba_",newnames[newnames %in% c(AbaseFeatures,baseFeatures)],sep="") 
+          newnames[newnames %in% bfeat] <- paste("Ba_",newnames[newnames %in% bfeat],sep="") 
           newnames[newnames %in% varincluded] <- paste("De_",newnames[newnames %in% varincluded],sep="")
           colnames(DeCorrmatrix) <- newnames
           newnames <- names(fscore)
-          newnames[newnames %in% c(AbaseFeatures,baseFeatures)] <- paste("Ba_",newnames[newnames %in% c(AbaseFeatures,baseFeatures)],sep="") 
+          newnames[newnames %in% bfeat] <- paste("Ba_",newnames[newnames %in% bfeat],sep="") 
           newnames[newnames %in% varincluded] <- paste("De_",newnames[newnames %in% varincluded],sep="")
           names(fscore) <- newnames
         }
@@ -540,6 +547,7 @@ getAllBetaCoefficients <- function(feat,varlist=NULL)
   attr(dataAdjusted,"GDSTM") <- DeCorrmatrix;
   attr(dataAdjusted,"varincluded") <- varincluded;
   attr(dataAdjusted,"baseFeatures") <- baseFeatures;
+  attr(dataAdjusted,"uniqueBase") <- bfeat;
   attr(dataAdjusted,"useDeCorr") <- useDeCorr;
   attr(dataAdjusted,"correlatedToBase") <- correlatedToBase;
   attr(dataAdjusted,"AbaseFeatures") <- AbaseFeatures;
@@ -554,11 +562,10 @@ predictDecorrelate <- function(decorrelatedobject,testData)
     decorMat <- attr(decorrelatedobject,"GDSTM")
 #    testData[,colnames(decorMat)] <- as.matrix(testData[,rownames(decorMat)]) %*% decorMat
     testData[,rownames(decorMat)] <- Rfast::mat.mult(as.matrix(testData[,rownames(decorMat)]),decorMat);
-    AbaseFeatures <- attr(decorrelatedobject,"AbaseFeatures")
-    baseFeatures <- attr(decorrelatedobject,"baseFeatures")
     varincluded <- attr(decorrelatedobject,"varincluded")
     newnames <- colnames(testData)
-    newnames[newnames %in% c(AbaseFeatures,baseFeatures)] <- paste("Ba_",newnames[newnames %in% c(AbaseFeatures,baseFeatures)],sep="") 
+    bfeat <- attr(decorrelatedobject,"uniqueBase")
+    newnames[newnames %in% bfeat] <- paste("Ba_",newnames[newnames %in% bfeat],sep="") 
     newnames[newnames %in% varincluded] <- paste("De_",newnames[newnames %in% varincluded],sep="")
     colnames(testData) <- newnames
   }
@@ -569,3 +576,25 @@ predictDecorrelate <- function(decorrelatedobject,testData)
   return (testData)
 }
 
+getDerivedCoefficients <- function(decorrelatedobject)
+{
+  CoeffList <- list();
+  nonZeronames <- character();
+  GDSTM <- attr(decorrelatedobject,"GDSTM")
+  namecol <- colnames(GDSTM);
+
+  n=1;
+  for (i in 1:ncol(GDSTM))
+  {
+    associ <- abs(GDSTM[,i])>0;
+    if (sum(associ) > 1)
+    {
+      nonZeronames <- append(nonZeronames,namecol[i]);
+      CoeffList[[n]] <- GDSTM[associ,i];
+      n=n+1;
+    }
+  }
+  names(CoeffList) <- nonZeronames;
+  
+  return (CoeffList)
+}
